/*******************************************************************************
 * Copyright (C) 2010  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 *******************************************************************************/
package org.ow2.orchestra.console.GWTConsole.client.Tab;

import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.event.dom.client.MouseOverEvent;
import com.google.gwt.event.dom.client.MouseOverHandler;
import com.google.gwt.user.client.ui.TabLayoutPanel;
import com.google.gwt.user.client.ui.Widget;
import org.ow2.orchestra.console.GWTConsole.client.Commons.PresenterIF;
import org.ow2.orchestra.console.GWTConsole.client.Resources.ImageResources;


/**
 * This class represents a tab of the tabLayoutPanel
 *
 * @author Jean-Yves Goubet
 *
 */
public class PTab extends PresenterIF {

  protected VTabIF view;

  private TabLayoutPanel tabLayoutPanel = null;

  private Widget widget = null;
  private PresenterIF presenterFils = null;


  public enum Type {
    LIST_PROCESS_ACTIVITY,  /* info. about a process & the process list activity */
    LIST_INSTANCE,          /* instance list of a process */

    LIST_INSTANCE_ACTIVITY,  /* info. about an instance & the instance list activity */

    FINDER,

    LIST_JOBS, /* info about jobs with exceptions */
    LIST_MESSAGES, /* info about pending messages */

    REPORTING_ALL,
    LIST_TASKS,
  };


  /**
   * The constructor of the class
   *
   * @author Jean-Yves Goubet
   * @param labelText : the text to display on the tab
   * @param processId : the id of the process to display instances'
   * @param tabLayoutPanel : the tab layout panel to add tabs
   * @param eventBus
   * @param currentTabPos
   */
  public PTab(String labelText, String elementId, Type type, Widget widget,
      PresenterIF presenter, TabLayoutPanel tabLayoutPanel) {
    this.tabLayoutPanel = tabLayoutPanel;
    this.widget = widget;
    this.presenterFils = presenter;
    VTab vTab = new VTab(elementId, labelText, type);
    this.bindingView(vTab);
    vTab.setTabLabel(labelText);

    // insert tab in the tab layout panel
    tabLayoutPanel.add(this.widget, vTab);
    this.setDebugID();
    this.addListeners();
  }

  protected void setDebugID() {

    if (this.view.getElementType() == PTab.Type.LIST_INSTANCE) {

      this.tabLayoutPanel.getTabWidget(
          this.tabLayoutPanel.getWidgetIndex(this.getWidget())
      ).ensureDebugId("tabLayoutPanelID-UUID-" + this.view.getElementId() + "-PE");

    } else if (this.view.getElementType() == PTab.Type.LIST_PROCESS_ACTIVITY) {

      this.tabLayoutPanel.getTabWidget(
          this.tabLayoutPanel.getWidgetIndex(this.getWidget())
      ).ensureDebugId("tabLayoutPanelID-UUID-" + this.view.getElementId() + "-PA");

    } else if (this.view.getElementType() == PTab.Type.LIST_INSTANCE_ACTIVITY) {

      this.tabLayoutPanel.getTabWidget(
          this.tabLayoutPanel.getWidgetIndex(this.getWidget())
      ).ensureDebugId("tabLayoutPanelID-UUID-" + this.view.getElementId() + "-IA");

    } /* else if (this.view.getElementType() == PTab.Type.FINDER) {

      this.tabLayoutPanel.getTabWidget(
          this.tabLayoutPanel.getWidgetIndex(this.getWidget())
      ).ensureDebugId("tabLayoutPanelID-FINDER");

    } */

  }

  protected void bindingView(final Object vTab) {
    this.view = (VTabIF) vTab;
  }

  /**
   * This method adds listeners on the image in the tab to close it.
   *
   * @author Jean-Yves Goubet
   */
  protected void addListeners() {
    this.view.getTabImage().addClickHandler(new ClickHandler() {
      public void onClick(ClickEvent arg0) {
        PTab.this.destroy();
        tabLayoutPanel.remove(PTab.this.widget);
      }
    });

    this.view.getTabImage().addMouseOverHandler(new MouseOverHandler() {
      public void onMouseOver(MouseOverEvent cursorPosition) {
        PTab.this.view.getTabImage().setResource(ImageResources.INSTANCE.closeOver());
        PTab.this.view.getTabImage().addStyleName("closeOverStyle");
        MouseOutHandlerForCloseTab mouseOutHandlerForCloseTab =
          new MouseOutHandlerForCloseTab(PTab.this.view.getTabImage());
        mouseOutHandlerForCloseTab.setHandlerRegistration(PTab.this.view.getTabImage().
            addMouseOutHandler(mouseOutHandlerForCloseTab));
      }
    });
  }


  /**
   * Get method of the widget child
   *
   * @author Jean-Yves Goubet
   * @param widget : the child widget attached
   */
  public Widget getWidget() {
    return this.widget;
  }

  public VTabIF getView() {
    return this.view;
  }



  public void destroy() {
    /*
     * possibilit� de refaire la m�me chose avec une interface Presenter qui obligerait tous les
     * presenter a impl�menter une method destroy & refresh
     */

    /*
    if (this.view.getElementType() == PTab.Type.LIST_INSTANCE) {
      if (presenterFils instanceof PListInstance) {
        ((PListInstance) presenterFils).destroy();
      }
    } else if (this.view.getElementType() == PTab.Type.LIST_PROCESS_ACTIVITY) {
      if (presenterFils instanceof PListProcessActivity) {
        ((PListProcessActivity) presenterFils).destroy();
      }
    } else if (this.view.getElementType() == PTab.Type.LIST_INSTANCE_ACTIVITY) {
      if (presenterFils instanceof PListInstanceActivity) {
        ((PListInstanceActivity) presenterFils).destroy();
      }
    } */

    presenterFils.destroy();
  }
}
