/**
 * Copyright (C) 2011  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.console.GWTConsole.client.userTask;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import com.google.gwt.cell.client.Cell;
import com.google.gwt.cell.client.CompositeCell;
import com.google.gwt.cell.client.HasCell;
import com.google.gwt.core.client.GWT;
import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.user.cellview.client.Column;
import com.google.gwt.user.cellview.client.ColumnSortEvent.ListHandler;
import com.google.gwt.user.cellview.client.IdentityColumn;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.view.client.ListDataProvider;
import org.ow2.orchestra.console.GWTConsole.client.Commons.celltable.ConsoleCellTable;
import org.ow2.orchestra.console.GWTConsole.client.Commons.celltable.ImageActionCell;
import org.ow2.orchestra.console.GWTConsole.client.Commons.celltable.LabelCell;
import org.ow2.orchestra.console.GWTConsole.client.Commons.celltable.NullableDateCell;
import org.ow2.orchestra.console.GWTConsole.client.ConsoleEventBus;
import org.ow2.orchestra.console.GWTConsole.client.Event.RefreshDoneEvent;
import org.ow2.orchestra.console.GWTConsole.client.Event.SuspendTimerEvent;
import org.ow2.orchestra.console.GWTConsole.client.Event.ViewRefreshEvent;
import org.ow2.orchestra.console.GWTConsole.client.Event.ViewRefreshHandler;
import org.ow2.orchestra.console.GWTConsole.client.OrchestraManagementException;
import org.ow2.orchestra.console.GWTConsole.client.OrchestraManagementServices;
import org.ow2.orchestra.console.GWTConsole.client.OrchestraManagementServicesAsync;
import org.ow2.orchestra.console.GWTConsole.client.Popup.PPopup;
import org.ow2.orchestra.console.GWTConsole.client.Resources.ImageResources;
import org.ow2.orchestra.console.GWTConsole.client.Tab.PTab.Type;


/**
 * @author Guillaume Porcher
 */
public class PListTasks {


  protected OrchestraManagementServicesAsync orchestramanagement = GWT.create(OrchestraManagementServices.class);

  // Create a data provider.
  private final ListDataProvider<MTask> dataProvider = new ListDataProvider<MTask>();

  // selected task
  private MTaskDetails selectedTaskDetails;

  // view
  private final VListTasks view;
  private final String debugPrefix = "listTasks";

  public PListTasks(final VListTasks view) {
    this.view = view;

    this.addViewRefreshHandlerToBus();

    view.getPager().setDisplay(view.getTable());
    view.getPager().ensureDebugId(debugPrefix + "Pager");

    //
    GWT.log("PListTasks - Constructor : PListTasks correctly instantiated");


    view.getTable().ensureDebugId(debugPrefix);

    // Add a ColumnSortEvent.ListHandler to connect sorting to the
    // java.util.List.
    final ListHandler<MTask> columnSortHandler = new ListHandler<MTask>(this.dataProvider.getList());
    view.getTable().addColumnSortHandler(columnSortHandler);

    this.setColumns(view.getTable(), columnSortHandler);

    // Connect the table to the data provider.
    this.dataProvider.addDataDisplay(view.getTable());

    // reset selected task
    this.setSelectedTask(null);

    // task detail button handlers
    this.view.getDetails().getStartButton().addClickHandler(new ClickHandler() {
      public void onClick(ClickEvent clickEvent) {
        PListTasks.this.orchestramanagement.startTask(PListTasks.this.selectedTaskDetails.getTaskId(), new AsyncCallback<Void>() {
          public void onFailure(Throwable caught) {
            GWT.log("TasksDetails - start: onFailure : " + caught.getMessage());
            ConsoleEventBus.getEventBus().fireEvent(new SuspendTimerEvent(true));
            if (caught instanceof OrchestraManagementException) {
              PPopup.launch(PPopup.PopupType.ErrorPopup, caught.getMessage(),
                  ((OrchestraManagementException) caught).getDetails());
            } else {
              PPopup.launch(PPopup.PopupType.ErrorPopup, "Unable to access to Orchestra !", caught.getMessage());
            }
          }

          public void onSuccess(Void aVoid) {
            PListTasks.this.refresh();
          }
        });
      }
    });
    this.view.getDetails().getCompleteButton().addClickHandler(new ClickHandler() {
      public void onClick(ClickEvent clickEvent) {
        PListTasks.this.orchestramanagement.completeTask(
            PListTasks.this.selectedTaskDetails.getTaskId(),
            PListTasks.this.view.getDetails().getCompleteTextBox().getText(),
            new AsyncCallback<Void>() {
              public void onFailure(Throwable caught) {
                GWT.log("TasksDetails - complete: onFailure : " + caught.getMessage());
                ConsoleEventBus.getEventBus().fireEvent(new SuspendTimerEvent(true));
                if (caught instanceof OrchestraManagementException) {
                  PPopup.launch(PPopup.PopupType.ErrorPopup, caught.getMessage(),
                      ((OrchestraManagementException) caught).getDetails());
                } else {
                  PPopup.launch(PPopup.PopupType.ErrorPopup, "Unable to access to Orchestra !", caught.getMessage());
                }
              }

              public void onSuccess(Void aVoid) {
                PListTasks.this.refresh();
              }
            });
      }
    });
    this.view.getDetails().getClaimButton().addClickHandler(new ClickHandler() {
      public void onClick(ClickEvent clickEvent) {
        PListTasks.this.orchestramanagement.claimTask(
            PListTasks.this.selectedTaskDetails.getTaskId(),
            new AsyncCallback<Void>() {
              public void onFailure(Throwable caught) {
                GWT.log("TasksDetails - claim: onFailure : " + caught.getMessage());
                ConsoleEventBus.getEventBus().fireEvent(new SuspendTimerEvent(true));
                if (caught instanceof OrchestraManagementException) {
                  PPopup.launch(PPopup.PopupType.ErrorPopup, caught.getMessage(),
                      ((OrchestraManagementException) caught).getDetails());
                } else {
                  PPopup.launch(PPopup.PopupType.ErrorPopup, "Unable to access to Orchestra !", caught.getMessage());
                }
              }

              public void onSuccess(Void aVoid) {
                PListTasks.this.refresh();
              }
            });
      }
    });
    this.view.getDetails().getStopButton().addClickHandler(new ClickHandler() {
      public void onClick(ClickEvent clickEvent) {
        PListTasks.this.orchestramanagement.stopTask(
            PListTasks.this.selectedTaskDetails.getTaskId(),
            new AsyncCallback<Void>() {
              public void onFailure(Throwable caught) {
                GWT.log("TasksDetails - stop: onFailure : " + caught.getMessage());
                ConsoleEventBus.getEventBus().fireEvent(new SuspendTimerEvent(true));
                if (caught instanceof OrchestraManagementException) {
                  PPopup.launch(PPopup.PopupType.ErrorPopup, caught.getMessage(),
                      ((OrchestraManagementException) caught).getDetails());
                } else {
                  PPopup.launch(PPopup.PopupType.ErrorPopup, "Unable to access to Orchestra !", caught.getMessage());
                }
              }

              public void onSuccess(Void aVoid) {
                PListTasks.this.refresh();
              }
            });
      }
    });
    this.view.getDetails().getSuspendButton().addClickHandler(new ClickHandler() {
      public void onClick(ClickEvent clickEvent) {
        PListTasks.this.orchestramanagement.suspendTask(
            PListTasks.this.selectedTaskDetails.getTaskId(),
            new AsyncCallback<Void>() {
              public void onFailure(Throwable caught) {
                GWT.log("TasksDetails - suspend: onFailure : " + caught.getMessage());
                ConsoleEventBus.getEventBus().fireEvent(new SuspendTimerEvent(true));
                if (caught instanceof OrchestraManagementException) {
                  PPopup.launch(PPopup.PopupType.ErrorPopup, caught.getMessage(),
                      ((OrchestraManagementException) caught).getDetails());
                } else {
                  PPopup.launch(PPopup.PopupType.ErrorPopup, "Unable to access to Orchestra !", caught.getMessage());
                }
              }

              public void onSuccess(Void aVoid) {
                PListTasks.this.refresh();
              }
            });
      }
    });
    this.view.getDetails().getResumeButton().addClickHandler(new ClickHandler() {
      public void onClick(ClickEvent clickEvent) {
        PListTasks.this.orchestramanagement.resumeTask(
            PListTasks.this.selectedTaskDetails.getTaskId(),
            new AsyncCallback<Void>() {
              public void onFailure(Throwable caught) {
                GWT.log("TasksDetails - resume: onFailure : " + caught.getMessage());
                ConsoleEventBus.getEventBus().fireEvent(new SuspendTimerEvent(true));
                if (caught instanceof OrchestraManagementException) {
                  PPopup.launch(PPopup.PopupType.ErrorPopup, caught.getMessage(),
                      ((OrchestraManagementException) caught).getDetails());
                } else {
                  PPopup.launch(PPopup.PopupType.ErrorPopup, "Unable to access to Orchestra !", caught.getMessage());
                }
              }

              public void onSuccess(Void aVoid) {
                PListTasks.this.refresh();
              }
            });
      }
    });
    this.view.getDetails().getReleaseButton().addClickHandler(new ClickHandler() {
      public void onClick(ClickEvent clickEvent) {
        PListTasks.this.orchestramanagement.releaseTask(
            PListTasks.this.selectedTaskDetails.getTaskId(),
            new AsyncCallback<Void>() {
              public void onFailure(Throwable caught) {
                GWT.log("TasksDetails - release: onFailure : " + caught.getMessage());
                ConsoleEventBus.getEventBus().fireEvent(new SuspendTimerEvent(true));
                if (caught instanceof OrchestraManagementException) {
                  PPopup.launch(PPopup.PopupType.ErrorPopup, caught.getMessage(),
                      ((OrchestraManagementException) caught).getDetails());
                } else {
                  PPopup.launch(PPopup.PopupType.ErrorPopup, "Unable to access to Orchestra !", caught.getMessage());
                }
              }

              public void onSuccess(Void aVoid) {
                PListTasks.this.refresh();
              }
            });
      }
    });
    // Fetch data
    this.refresh();

  }

  private void setColumns(final ConsoleCellTable<MTask> table, final ListHandler<MTask> columnSortHandler) {

    /* Name column */
    final Column<MTask, String> nameColumn = new Column<MTask, String>(new LabelCell(this.debugPrefix)) {
      @Override
      public String getValue(final MTask mTask) {
        return mTask.getName();
      }
    };
    // Add the columns.
    table.addSortedColumn(columnSortHandler, nameColumn, "Name");

    /* Due date column */
    final Column<MTask, Date> createdDateColumn = new Column<MTask, Date>(new NullableDateCell(this.debugPrefix)) {
      @Override
      public Date getValue(final MTask mTask) {
        return mTask.getCreatedDate();
      }
    };
    // Add the columns.
    table.addSortedColumn(columnSortHandler, createdDateColumn, "Creation Date");

    /* ID column */
    final Column<MTask, String> idColumn = new Column<MTask, String>(new LabelCell(this.debugPrefix)) {
      @Override
      public String getValue(final MTask mTask) {
        return mTask.getTaskId();
      }
    };
    // Add the columns.
    table.addSortedColumn(columnSortHandler, idColumn, "ID");

    /* Type column */
    final Column<MTask, String> typeColumn = new Column<MTask, String>(new LabelCell(this.debugPrefix)) {
      @Override
      public String getValue(final MTask mTask) {
        return mTask.getTaskType();
      }
    };
    // Add the columns.
    table.addSortedColumn(columnSortHandler, typeColumn, "Type");

    /* Status column */
    final Column<MTask, String> statusColumn = new Column<MTask, String>(new LabelCell(this.debugPrefix)) {
      @Override
      public String getValue(final MTask mTask) {
        return mTask.getStatus();
      }
    };
    // Add the columns.
    table.addSortedColumn(columnSortHandler, statusColumn, "Status");

    /* Actions column */
    final List<HasCell<MTask, ?>> actionList = new ArrayList<HasCell<MTask, ?>>();
    actionList.add(new IdentityColumn<MTask>(this.getTaskDetailsActionCell()));

    final CompositeCell<MTask> actionCells = new CompositeCell<MTask>(actionList);
    final Column<MTask, MTask> actionsColumn = new IdentityColumn<MTask>(actionCells);
    actionsColumn.setSortable(false);
    // Add the columns.
    table.addColumn(actionsColumn, "Actions");

  }

  private Cell<MTask> getTaskDetailsActionCell() {
    final Cell<MTask> taskDetailsAction = new ImageActionCell<MTask>(ImageResources.INSTANCE.activities(),
        ImageResources.INSTANCE.activitiesDisabled(), "info", this.debugPrefix + "-info") {

      @Override
      public void execute(final MTask object) {
        // show task details
        PListTasks.this.setSelectedTask(object);
      }

      @Override
      public boolean isActive(final MTask value) {
        return true;
      }
    };
    return taskDetailsAction;
  }

  public void refresh() {
    this.orchestramanagement.getTasks(new AsyncCallback<List<MTask>>() {
      public void onSuccess(final List<MTask> tasks) {
        PListTasks.this.dataProvider.getList().clear();
        PListTasks.this.dataProvider.getList().addAll(tasks);
        if (PListTasks.this.selectedTaskDetails != null) {
          String taskId = PListTasks.this.selectedTaskDetails.getTaskId();
          MTask task = null;
          for (MTask t : tasks) {
            if (t.getTaskId().equals(taskId)) {
              task = t;
              break;
            }
          }
          PListTasks.this.setSelectedTask(task);
        }
        ConsoleEventBus.getEventBus().fireEvent(new RefreshDoneEvent());
      }

      public void onFailure(final Throwable caught) {
        GWT.log("TasksCellTable - refreshCurrentList: onFailure : " + caught.getMessage());
        ConsoleEventBus.getEventBus().fireEvent(new SuspendTimerEvent(true));
        if (caught instanceof OrchestraManagementException) {
          PPopup.launch(PPopup.PopupType.ErrorPopup, caught.getMessage(),
              ((OrchestraManagementException) caught).getDetails());
        } else {
          PPopup.launch(PPopup.PopupType.ErrorPopup, "Unable to access to Orchestra !", caught.getMessage());
        }
      }
    });
  }

  /**
   * Wait refresh event on the eventBus to launch refreshCurrentList
   *
   * @author Jean-Yves Goubet
   */
  private void addViewRefreshHandlerToBus() {
    ConsoleEventBus.getEventBus().addHandler(ViewRefreshEvent.TYPE, new ViewRefreshHandler() {

      public void onRefresh(final ViewRefreshEvent viewRefreshEvent) {
        if (viewRefreshEvent.getElementType() == Type.LIST_TASKS) {
          PListTasks.this.refresh();
        }
      }
    });
  }

  public void setSelectedTask(MTask selectedTask) {
    PListTasks.this.view.getDetails().setVisible(selectedTask != null);
    if (selectedTask != null) {
      this.orchestramanagement.getTaskDetails(selectedTask.getTaskId(), new AsyncCallback<MTaskDetails>() {
        public void onSuccess(final MTaskDetails taskDetails) {
          PListTasks.this.selectedTaskDetails = taskDetails;
          PListTasks.this.view.getDetails().setVisible(taskDetails != null);

          if (taskDetails != null) {
            PListTasks.this.view.getDetails().getId().setText("Task " + taskDetails.getTaskId());
            PListTasks.this.view.getDetails().getName().setText(taskDetails.getName());
            PListTasks.this.view.getDetails().getSubject().setText(taskDetails.getSubject());
            PListTasks.this.view.getDetails().getDescription().setText(taskDetails.getDescription());
            PListTasks.this.view.getDetails().getActualOwner().setText(taskDetails.getActualOwner());
            PListTasks.this.view.getDetails().getPotentialOwners().setText(taskDetails.getPotentialOwners().toString());
            PListTasks.this.view.getDetails().getStatus().setText(taskDetails.getStatus().toLowerCase());
            if (taskDetails.getStatus().equals("READY")) {
              PListTasks.this.view.getDetails().getClaimButton().setVisible(true);
              PListTasks.this.view.getDetails().getReleaseButton().setVisible(false);
              PListTasks.this.view.getDetails().getStopButton().setVisible(false);
              PListTasks.this.view.getDetails().getStartButton().setVisible(true);
              PListTasks.this.view.getDetails().getCompleteTextBox().setVisible(false);
              PListTasks.this.view.getDetails().getCompleteButton().setVisible(false);
              PListTasks.this.view.getDetails().getSuspendButton().setVisible(true);
              PListTasks.this.view.getDetails().getResumeButton().setVisible(false);
            } else if (taskDetails.getStatus().equals("RESERVED")) {
              PListTasks.this.view.getDetails().getClaimButton().setVisible(false);
              PListTasks.this.view.getDetails().getReleaseButton().setVisible(true);
              PListTasks.this.view.getDetails().getStartButton().setVisible(true);
              PListTasks.this.view.getDetails().getStopButton().setVisible(false);
              PListTasks.this.view.getDetails().getCompleteTextBox().setVisible(false);
              PListTasks.this.view.getDetails().getCompleteButton().setVisible(false);
              PListTasks.this.view.getDetails().getSuspendButton().setVisible(true);
              PListTasks.this.view.getDetails().getResumeButton().setVisible(false);
            } else if (taskDetails.getStatus().equals("IN_PROGRESS")) {
              PListTasks.this.view.getDetails().getClaimButton().setVisible(false);
              PListTasks.this.view.getDetails().getReleaseButton().setVisible(true);
              PListTasks.this.view.getDetails().getStartButton().setVisible(false);
              PListTasks.this.view.getDetails().getStopButton().setVisible(true);
              PListTasks.this.view.getDetails().getCompleteTextBox().setVisible(true);
              PListTasks.this.view.getDetails().getCompleteButton().setVisible(true);
              PListTasks.this.view.getDetails().getSuspendButton().setVisible(true);
              PListTasks.this.view.getDetails().getResumeButton().setVisible(false);
            } else if (taskDetails.getStatus().equals("SUSPENDED")) {
              PListTasks.this.view.getDetails().getClaimButton().setVisible(false);
              PListTasks.this.view.getDetails().getReleaseButton().setVisible(false);
              PListTasks.this.view.getDetails().getStartButton().setVisible(false);
              PListTasks.this.view.getDetails().getStopButton().setVisible(false);
              PListTasks.this.view.getDetails().getCompleteTextBox().setVisible(false);
              PListTasks.this.view.getDetails().getCompleteButton().setVisible(false);
              PListTasks.this.view.getDetails().getSuspendButton().setVisible(false);
              PListTasks.this.view.getDetails().getResumeButton().setVisible(true);
            } else {
              PListTasks.this.view.getDetails().getClaimButton().setVisible(false);
              PListTasks.this.view.getDetails().getReleaseButton().setVisible(false);
              PListTasks.this.view.getDetails().getStartButton().setVisible(false);
              PListTasks.this.view.getDetails().getStopButton().setVisible(false);
              PListTasks.this.view.getDetails().getCompleteTextBox().setVisible(false);
              PListTasks.this.view.getDetails().getCompleteButton().setVisible(false);
              PListTasks.this.view.getDetails().getSuspendButton().setVisible(false);
              PListTasks.this.view.getDetails().getResumeButton().setVisible(false);
            }
          }
        }

        public void onFailure(final Throwable caught) {
          GWT.log("Tasks - select task: onFailure : " + caught.getMessage());
          ConsoleEventBus.getEventBus().fireEvent(new SuspendTimerEvent(true));
          if (caught instanceof OrchestraManagementException) {
            PPopup.launch(PPopup.PopupType.ErrorPopup, caught.getMessage(),
                ((OrchestraManagementException) caught).getDetails());
          } else {
            PPopup.launch(PPopup.PopupType.ErrorPopup, "Unable to access to Orchestra !", caught.getMessage());
          }
        }
      });
    }
  }
}
